@echo off
@rem ==========================================================================
@rem == Licensed Materials - Property of IBM Corporation
@rem == (C) Copyright IBM Corporation 2018
@rem ==========================================================================
@rem Deploys patch files for the installed packs.
@rem Run this from an SPE console window. This is a WINDOWS ONLY script.
@rem ==========================================================================
@rem
if not defined InSPEConsole echo This command must be run in an SPE console session
if not defined InSPEConsole goto :EOF

Setlocal EnableDelayedExpansion

for /f "delims=" %%a in ('wmic OS Get localdatetime  ^| find "."') do set dt=%%a
set TIMESTAMP=D%dt:~0,8%.T%dt:~8,6%
set PACK_V_R_M=
set WIN_JAR=
set REF_ID=DEFAULT
set updateDB=TRUE
set updateDBArgument=db
set JAR_WTX=D3ADB33F
set resourceType=patch

:READ_COMMAND_LINE_PARMS
if "%1"=="" goto VALIDATE_WIN_JAR

rem read in -j
if /i "%1"=="-j" (
rem read in the -j value: \path\to\jar
    set WIN_JAR=%2
    shift
    goto READ_NEXT
)

rem read in -t
if /i "%1"=="-t" (
rem read in the optional -t value: tenant (e.g., DEFAULT)
    set REF_ID=%2
    if /i "!REF_ID:~0,1!"=="-" goto READ_NEXT
    shift
    goto READ_NEXT
)

rem read in -u
if /i "%1"=="-u" (
rem read in the optional -u value: db | nodb
    set updateDBArgument=%2
    if /i "!updateDBArgument:~0,1!"=="-" goto READ_NEXT
    shift
    goto READ_NEXT
)

rem help syntax
if /i "%1"=="/?" goto SYNTAX

set MESSAGE=Invalid option %1
echo %MESSAGE%
echo.
goto SYNTAX

:READ_NEXT
shift
goto READ_COMMAND_LINE_PARMS

:VALIDATE_WIN_JAR
echo WIN_JAR=%WIN_JAR%
set MESSAGE=Missing mandatory parameter -j
if not defined WIN_JAR goto ERROR
if /i '%WIN_JAR%'=='nojar' goto VALIDATE_REF_ID
set MESSAGE=Resource jar file does not exist: %WIN_JAR%
if not exist %WIN_JAR% goto ERROR
rem extract the last four characters from the jar name
set JAR_EXT=%WIN_JAR:~-4%
rem remove the last character. it might be a double quote
set JAR_EXT=%JAR_EXT:~0,-1%
set MESSAGE=Invalid jar file name extension: %JAR_EXT%
rem if jar path was specified with double quotes
if /i "%JAR_EXT%"=="jar" goto VALIDATE_REF_ID
rem if jar path was not specified with double quotes
if /i "%JAR_EXT%"==".ja" goto VALIDATE_REF_ID
goto ERROR

:VALIDATE_REF_ID
echo tenantID=%REF_ID%
set MESSAGE=Missing tenantID value for parameter -t
if not defined REF_ID goto ERROR
set MESSAGE=Please specify a valid tenantID after the -t option
if /i "%REF_ID:~0,1%"=="-" goto ERROR

:VALIDATE_UPDATE_DB
echo.
set MESSAGE=Missing db or nodb value for parameter -u
if not defined updateDBArgument goto ERROR
set MESSAGE=Please specify db or nodb value after the -u option
if /i "%updateDBArgument%"=="db" (
    set updateDB=TRUE
    echo updateDB=!updateDB!
    goto VALIDATE_OK
)
if /i "%updateDBArgument%"=="nodb" (
    set updateDB=FALSE
    echo updateDB=!updateDB!
    goto VALIDATE_OK
)
goto ERROR

:VALIDATE_OK
rem Generate WORK_DIR name and extract jar to it
for %%j in (%WIN_JAR%) do (
    Set JAR_NAME=%%~nxj
)
echo JAR_NAME=%JAR_NAME%
set NO_JAR_NAME=%JAR_NAME:~0,-4%
set WORK_DIR="%SPE_INSTALL_DIR%\%NO_JAR_NAME%"
echo WORK_DIR=%WORK_DIR%
set MESSAGE=Failed to remove %WORK_DIR%
if exist %WORK_DIR% rd /s /q %WORK_DIR%
if ERRORLEVEL 1 goto ERROR

echo.
echo Extracting files...
set MESSAGE=Failed to extract %WIN_JAR% to %WORK_DIR%
java com.ibm.spe.core.utils.ExtractJarFile -name=%WIN_JAR% -output=%WORK_DIR%
if ERRORLEVEL 1 goto ERROR
set MESSAGE=Failed to change directory to %WORK_DIR%
cd /d %WORK_DIR%
if ERRORLEVEL 1 goto ERROR

echo.
echo Verifying the OS using the UNAME file
set localOS=
for /f "tokens=2 usebackq" %%w in (`ver`) do (
    set localOS=%%w
)
set MESSAGE=Missing the UNAME file. Cannot determine the target platform
if not exist UNAME goto ERROR
set targetOS=
set /p targetOS=<UNAME
echo Target OS=[%targetOS%]
echo Local OS=[%localOS%]
set MESSAGE=Could not find a match for Local OS in Target OS
echo %targetOS% | findstr %localOS% > nul
if ERRORLEVEL 1 goto ERROR
echo Local OS matches the target one

cd /d "%WORK_DIR%"
set MESSAGE=Missing the PACK file. Cannot determine the pack type
if not exist PACK goto ERROR
set /p PACK=<PACK

set MESSAGE=Missing the PACK_VERSION file. Cannot determine the ${PACK} target version
if not exist PACK_VERSION goto ERROR
set /p PACK_VERSION=<PACK_VERSION 
echo Bundle Jar Version=[%PACK_VERSION%]
set PACK_V_R_M=%PACK_VERSION:~0,5%

echo.
echo Verifying the pack version using the PACK_VERSION
set PROP_VER=properties/version
set EDI_SWTAG_FILE=ibm.com_IBM_Transformation_Extender_Pack_for_Supply_Chain_EDI_Server-%PACK_V_R_M%.swidtag
set HC_SWTAG_FILE=ibm.com_IBM_Transformation_Extender_Pack_for_Healthcare_server-%PACK_V_R_M%.swidtag
set FSP_SWTAG_FILE=ibm.com_IBM_Transformation_Extender_Pack_for_Financial_Payments_server-%PACK_V_R_M%.swidtag

set MESSAGE=The %PACK% %PACK_V_R_M% pack is not installed, you cannot deploy this patch bundle
if "%PACK%"=="edi" if not exist "%SPE_INSTALL_DIR%\%PROP_VER%\%EDI_SWTAG_FILE%" goto ERROR
if "%PACK%"=="hc" if not exist "%SPE_INSTALL_DIR%\%PROP_VER%\%HC_SWTAG_FILE%" goto ERROR
if "%PACK%"=="fsp" if not exist "%SPE_INSTALL_DIR%\%PROP_VER%\%FSP_SWTAG_FILE%" goto ERROR

:VALIDATE_RESOURCE_TYPE
echo.
echo Verifying the resource type using the RESOURCE file
set MESSAGE=Missing the RESOURCE file. Cannot determine the resource type
if not exist RESOURCE goto ERROR
set resourceType=
set /p resourceType=<RESOURCE
set resourceType=%resourceType:~0,5%
echo Resource bundle=[%resourceType%]
echo Expected bundle=[patch]
set MESSAGE=Invalid resource type %resourceType%
if not "%resourceType%"=="patch" goto ERROR

:DEPLOY_BUNDLE
cd /d "%WORK_DIR%"
echo.
echo Deploying %PACK% %PACK_V_R_M% %resourceType%
echo.
echo Removing accessory files
del /q /f /s PACK
del /q /f /s PACK_VERSION
del /q /f /s RESOURCE
del /q /f /s UNAME
rd  /q /s META-INF

echo.
set BACKUP_DIR=backups\PRE-%NO_JAR_NAME%-PATCH-%TIMESTAMP%
echo Creating %BACKUP_DIR% directories...
set MESSAGE=Failed to create directories
for /d /r %%d in (*) do (
    set line=%%d  
    if not exist "%SPE_INSTALL_DIR%\%BACKUP_DIR%!line:%CD%=!" (
        md "%SPE_INSTALL_DIR%\%BACKUP_DIR%!line:%CD%=!"
        if ERRORLEVEL 1 goto ERROR 
    )
)

echo.
rem Get list of files to copy 
set LIST_OF_FILES="%SPE_INSTALL_DIR%\%NO_JAR_NAME%.txt"
if exist %LIST_OF_FILES% del /q /f /s %LIST_OF_FILES%
for /r %%f in (*) do (
    set line=%%f
    echo .!line:%CD%=! >> %LIST_OF_FILES%
)

echo.
echo Deploying files...
set FILE=
set TARGET_FILE=
set MESSAGE=Failed to deploy files
for /f "tokens=1* delims=]" %%i in ('find /n /v ""^<%LIST_OF_FILES%') do (
    set FILE=%%j
    set FILE=!FILE:~0,-1!
    set TARGET_FILE=%SPE_INSTALL_DIR%\!FILE!
    set BACKUP_FILE=%SPE_INSTALL_DIR%\%BACKUP_DIR%\!FILE!
    echo backing up old !FILE!
    move "!TARGET_FILE!" "!BACKUP_FILE!"
    if ERRORLEVEL 1 goto ERROR     
    echo deploying !FILE!
    copy /y "!FILE!" "!TARGET_FILE!"
    if ERRORLEVEL 1 goto ERROR 
)

set DBPROPS="dbprops=%SPE_INSTALL_DIR%\dbprops.cfg"

rem Import pack resources
cd /d %WORK_DIR%

:IMPORT_EDI_RESOURCES
if /i "%PACK%"=="edi" (
    echo.
    if /i '%updateDB%'=='TRUE' (
        if exist "%SPE_INSTALL_DIR%\schema\SPE_IE_Standard_Types_X12.xsd" (
            set MESSAGE=There was an error while loading "%SPE_INSTALL_DIR%\schema\SPE_IE_Standard_Types_X12.xsd"
            java com.ibm.spe.core.utils.SPEImporter %DBPROPS% quiet updateExisting updateDefaultVersion -xsd="%SPE_INSTALL_DIR%\schema\SPE_IE_Standard_Types_X12.xsd" refId=%REF_ID%
            if ERRORLEVEL 1 goto ERROR    

        )
    )
)    

:IMPORT_HC_RESOURCES
if /i "%PACK%"=="hc" (
    echo.
)

:IMPORT_FSP_RESOURCES
if /i "%PACK%"=="fsp" (
    echo.
)

goto DONE

:DONE
set MESSAGE=
cd /d %SPE_INSTALL_DIR%
echo.
if exist %WORK_DIR% rd /s /q %WORK_DIR%
if exist %LIST_OF_FILES% del /q /f %LIST_OF_FILES%
echo Pack files were deployed, configured, and loaded successfully
echo.
goto :EOF

:ERROR
echo.
echo.%MESSAGE%
echo.
cd /d %SPE_INSTALL_DIR%
rem DO NOT REMOVE THE WORK_DIR FOR PROBLEM DETERMINATION
if exist %LIST_OF_FILES% del /q /f %LIST_OF_FILES%
goto :EOF

:SYNTAX
echo Deploys pack %resourceType% resources
echo.
echo %0 -j \path\to\jar [ -t tenantID ] [ -u db^|nodb ]
echo.
echo Command line arguments:
echo -j  (Required) Full path to the resource jar file
echo -t  (Optional) Tenant ID under which the resources should be imported.
echo     It defaults to the 'DEFAULT' tenant if not specified.  
echo -u  (Optional) The 'db' value indicates that the common resources need to be imported.
echo     It defaults to the 'db' value if not specified. Specify 'nodb' to skip the import.
echo.
goto :EOF

:EOF
